@@ -5,6 +5,9 @@ tag="$2"
 
 test -n "$tag" || { echo "usage: $0 PY TAG"; exit 1; }
 
+PYTHONIOENCODING=utf8; export PYTHONIOENCODING
+#PYTHONUTF8=1; export PYTHONUTF8
+
 mkdir -p tmp
 diffs="tmp/output.$tag.diffs"
 rm -f "$diffs"
@@ -30,17 +33,20 @@ for f in test/files/*.rar; do
     rm -f "$f.$tag"
   else
     $vecho "FAIL"
+    errmsg="FAILED"
+    case "$f" in
+        *-hpsw.rar) errmsg="failed-nocrypto";;
+        *) result=1;;
+    esac
     if test -n "$quiet"; then
-      printf "[%s] %-30s .. FAILED\n" "$tag" "$f"
+      printf "[%s] %-30s .. ${errmsg}\n" "$tag" "$f"
     fi
     echo "#### $py ####" >> "$diffs"
     diff -uw "$f.exp" "$f.$tag" >> "$diffs"
-    case "$f" in
-        *-hpsw.rar) ;;
-        *) result=1;;
-    esac
   fi
 done
 
+test "$result" = "0" || echo "Diffs: ${diffs}"
+
 exit $result
 
